(function() {
    if (window.WebViewJavascriptBridge) {
        return;
    }
    var messageHandlers = {};
    var responseCallbacks = {};
    var errorResponseCallbacks = {};

    function exec(handlerName, actionName, data, responseCallback, errorResponseCallback) {
            var msg = null;
            if (handlerName) {
                msg = {handlerName: handlerName, actionName: actionName, data: data};
            }

            if (msg) {
                if (responseCallback || errorResponseCallback) {
                    var jsCallbackId = "callback_js_" + new Date().getTime();
                    msg.jsCallbackId = jsCallbackId;
                    if (responseCallback) {
                        responseCallbacks[jsCallbackId] = responseCallback;
                    }

                    if (errorResponseCallback) {
                        errorResponseCallbacks[jsCallbackId] = errorResponseCallback;
                    }
                }
            }

            var msgStr = JSON.stringify(msg);
            bridge_wk.handleMessageFromJs(msgStr);
    }

    //提供给native使用,
    function _handleMessageFromNative(messageJSON) {
        var message = JSON.parse(messageJSON);
        var responseCallback;
        //java call finished, now need to call js callback function
        if (message.jsCallbackId) {
            if (message.isSuccess) {
                responseCallback = responseCallbacks[message.jsCallbackId];
                if (responseCallback) {
                    responseCallback(message.data);
                }
            } else {
                errorResponseCallback = errorResponseCallbacks[message.jsCallbackId];
                if (errorResponseCallback) {
                    errorResponseCallback(message.data);
                }
            }
            delete responseCallbacks[message.jsCallbackId];
            delete errorResponseCallbacks[message.jsCallbackId];
        } else {
            //直接发送

            var errorResponseCallback;
            if (message.nativeCallbackId) {
                var nativeCallbackId = message.nativeCallbackId;
                responseCallback = function(jsResponseSuccessData) {

                    var jsResponseSuccessMessage = {
                        nativeCallbackId: nativeCallbackId,
                        isSuccess: true,
                        data: jsResponseSuccessData
                    };
                    var msgStr = JSON.stringify(jsResponseSuccessMessage);
                    bridge_wk.handleMessageFromJs(msgStr);
                };

                errorResponseCallback = function(jsResponseFailedData) {
                    var jsResponseErrorMessage = {
                        nativeCallbackId: nativeCallbackId,
                        isSuccess: false,
                        data: jsResponseFailedData
                    };
                    var msgStr = JSON.stringify(jsResponseErrorMessage);
                    bridge_wk.handleMessageFromJs(msgStr);
                };

            }

            var handler = WebViewJavascriptBridge._defaultMessageHandler;
            if (message.handlerName) {
                handler = messageHandlers[message.handlerName];
            }
            //查找指定handler
            try {
                handler(message.data, responseCallback, errorResponseCallback);
            } catch (exception) {
                if (typeof console != 'undefined') {
                    console.log("WebViewJavascriptBridge: WARNING: javascript handler threw.", message, exception);
                }
            }
        }
    }

    //set default messageHandler
    function init(defaultMessageHandler) {
        if (WebViewJavascriptBridge._defaultMessageHandler) {
            throw new Error("bridge.init() called twice");
        }
        WebViewJavascriptBridge._defaultMessageHandler = defaultMessageHandler;
    }

    function registerHandler(handlerName, handler) {
        messageHandlers[handlerName] = handler;
    }


    window.WebViewJavascriptBridge = {
        init: init,
        registerHandler: registerHandler,
        _handleMessageFromNative: _handleMessageFromNative,
        exec: exec
    };

    var doc = document;
    var readyEvent = doc.createEvent('Events');
    readyEvent.initEvent('WebViewJavascriptBridgeReady');
    readyEvent.bridge = WebViewJavascriptBridge;
    doc.dispatchEvent(readyEvent);
})();